@@ -26,11 +26,8 @@
 #include <ctype.h>
 #include <unistd.h>
 #ifdef HAVE_DOSISH_SYSTEM
-  #include <fcntl.h> /* for setmode() */
+#include <fcntl.h> /* for setmode() */
 #endif
-#ifdef __riscos__
-#include <unixlib/local.h>
-#endif /* __riscos__ */
 
 #define INCLUDED_BY_MAIN_MODULE 1
 #include "packet.h"
@@ -81,6 +78,10 @@ static ARGPARSE_OPTS opts[] = {
 
 int g10_errors_seen = 0;
 
+#ifdef __riscos__
+RISCOS_GLOBAL_STATICS("GnuPG (gpgv) Heap")
+#endif /* __riscos__ */
+
 const char *
 strusage( int level )
 {
@@ -108,25 +109,36 @@ strusage( int level )
 }
 
 
+/* We need the home directory also in some other directories, so make
+   sure that both variables are always in sync. */
+static void
+set_homedir (char *dir)
+{
+  if (!dir)
+    dir = "";
+  g10_opt_homedir = opt.homedir = dir;
+}
+
+
 
 
 static void
 i18n_init(void)
 {
-  #ifdef USE_SIMPLE_GETTEXT
+#ifdef USE_SIMPLE_GETTEXT
     set_gettext_file( PACKAGE );
-  #else
-  #ifdef ENABLE_NLS
-    #ifdef HAVE_LC_MESSAGES
+#else
+#ifdef ENABLE_NLS
+#ifdef HAVE_LC_MESSAGES
        setlocale( LC_TIME, "" );
        setlocale( LC_MESSAGES, "" );
-    #else
+#else
        setlocale( LC_ALL, "" );
-    #endif
+#endif
     bindtextdomain( PACKAGE, G10_LOCALEDIR );
     textdomain( PACKAGE );
-  #endif
-  #endif
+#endif
+#endif
 }
 
 
@@ -138,9 +150,10 @@ main( int argc, char **argv )
     STRLIST sl;
     STRLIST nrings=NULL;
     unsigned configlineno;
-  #ifdef __riscos__
-    __riscosify_control = __RISCOSIFY_NO_PROCESS;
-  #endif /* __riscos__ */
+
+#ifdef __riscos__
+    riscos_global_defaults();
+#endif /* __riscos__ */
 
     log_set_name("gpgv");
     init_signals();
@@ -151,17 +164,19 @@ main( int argc, char **argv )
     opt.always_trust = 1;
     opt.batch = 1;
 
-  #if defined (__MINGW32__) || defined (__CYGWIN32__)
-    opt.homedir = read_w32_registry_string( NULL, "Software\\GNU\\GnuPG", "HomeDir" );
-  #else
-    opt.homedir = getenv("GNUPGHOME");
-  #endif
-    if( !opt.homedir || !*opt.homedir ) {
-	opt.homedir = GNUPG_HOMEDIR;
+#if defined (_WIN32)
+    set_homedir (read_w32_registry_string( NULL, "Software\\GNU\\GnuPG", "HomeDir" ));
+#else
+    set_homedir (getenv("GNUPGHOME"));
+#endif
+    if( !*opt.homedir ) {
+      set_homedir (GNUPG_HOMEDIR);
     }
     tty_no_terminal(1);
     tty_batchmode(1);
     disable_dotlock();
+
+    set_native_charset (NULL); /* Try to auto set the character set */
     
     pargs.argc = &argc;
     pargs.argv = &argv;
@@ -174,7 +189,8 @@ main( int argc, char **argv )
           case oKeyring: append_to_strlist( &nrings, pargs.r.ret_str); break;
 	  case oStatusFD: set_status_fd( pargs.r.ret_int ); break;
 	  case oLoggerFD: log_set_logfile( NULL, pargs.r.ret_int ); break;
-	  case oHomedir: opt.homedir = pargs.r.ret_str; break;
+	  case oHomedir: set_homedir (pargs.r.ret_str); break;
+	  case oIgnoreTimeConflict: opt.ignore_time_conflict = 1; break;
 	  default : pargs.err = 2; break;
 	}
     }
@@ -182,8 +198,6 @@ main( int argc, char **argv )
     if( log_get_errorcount(0) )
 	g10_exit(2);
 
-    g10_opt_homedir = opt.homedir;
-
     if( opt.verbose > 1 )
 	set_packet_list_mode(1);
 
@@ -222,13 +236,22 @@ check_signatures_trust( PKT_signature *sig )
     return 0;
 }
 
+void
+read_trust_options(byte *trust_model,ulong *created,ulong *nextcheck) {}
 
 /* Stub: 
  * We don't have the trustdb , so we have to provide some stub functions
  * instead
  */
+
 int
-get_validity_info (PKT_public_key *pk, const byte *namehash )
+cache_disabled_value(PKT_public_key *pk)
+{
+  return 0;
+}
+
+int
+get_validity_info (PKT_public_key *pk, PKT_user_id *uid)
 {
     return '?';
 }
@@ -240,6 +263,12 @@ get_ownertrust_info (PKT_public_key *pk)
     return '?';
 }
 
+unsigned int
+get_ownertrust (PKT_public_key *pk)
+{
+    return TRUST_UNKNOWN;
+}
+
 
 /* Stub:
  * Because we only work with trusted keys, it does not make sense to
@@ -296,13 +325,17 @@ check_secret_key( PKT_secret_key *sk, int n )
 DEK *
 passphrase_to_dek( u32 *keyid, int pubkey_algo,
 		   int cipher_algo, STRING2KEY *s2k, int mode,
-                   const char *tmp)
+                   const char *tmp, int *canceled)
 {
-    return NULL;
+  if (canceled)
+    *canceled = 0;
+  return NULL;
 }
 
-/* Stub to avoid linking to photoid.c */
-void show_photo(const struct user_attribute *attr,PKT_public_key *pk) {}
+/* Stubs to avoid linking to photoid.c */
+void show_photos(const struct user_attribute *attrs,int count,PKT_public_key *pk) {}
+int parse_image_header(const struct user_attribute *attr,byte *type,u32 *len) {return 0;}
+char *image_type_to_string(byte type,int string) {return NULL;}
 
 /* Stubs to void linking to ../cipher/cipher.c */
 int string_to_cipher_algo( const char *string ) { return 0; }
