@@ -1,5 +1,6 @@
 /* sign.c - sign data
- * Copyright (C) 1998, 1999, 2000, 2001, 2002 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001, 2002,
+ *               2003 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -42,11 +43,11 @@
 
 
 #ifdef HAVE_DOSISH_SYSTEM
-  #define LF "\r\n"
-  void __stdcall Sleep(ulong);
-  #define sleep(a)  Sleep((a)*1000)
+#define LF "\r\n"
+void __stdcall Sleep(ulong);
+#define sleep(a)  Sleep((a)*1000)
 #else
-  #define LF "\n"
+#define LF "\n"
 #endif
 
 static int recipient_digest_algo=0;
@@ -71,20 +72,25 @@ mk_notation_and_policy( PKT_signature *sig,
     args.pk=pk;
     args.sk=sk;
 
+    /* It is actually impossible to get here when making a v3 key
+       signature since keyedit.c:sign_uids will automatically bump a
+       signature with a notation or policy url up to v4, but it is
+       good to do these checks anyway. */
+
     /* notation data */
     if(IS_SIG(sig) && opt.sig_notation_data)
       {
 	if(sig->version<4)
-	  log_info(_("can't put notation data into v3 (PGP 2.x style) "
-		     "signatures\n"));
+	  log_error(_("can't put notation data into v3 (PGP 2.x style) "
+		      "signatures\n"));
 	else
 	  nd=opt.sig_notation_data;
       }
     else if( IS_CERT(sig) && opt.cert_notation_data )
       {
 	if(sig->version<4)
-	  log_info(_("can't put notation data into v3 (PGP 2.x style) "
-		     "key signatures\n"));
+	  log_error(_("can't put notation data into v3 (PGP 2.x style) "
+		      "key signatures\n"));
 	else
 	  nd=opt.cert_notation_data;
       }
@@ -124,23 +130,20 @@ mk_notation_and_policy( PKT_signature *sig,
 	m_free(buf);
     }
 
-    if(opt.show_notation)
-      show_notation(sig,0);
-
     /* set policy URL */
     if( IS_SIG(sig) && opt.sig_policy_url )
       {
 	if(sig->version<4)
-	  log_info(_("can't put a policy URL into v3 (PGP 2.x style) "
-		     "signatures\n"));
+	  log_error(_("can't put a policy URL into v3 (PGP 2.x style) "
+		      "signatures\n"));
 	else
 	  pu=opt.sig_policy_url;
       }
     else if( IS_CERT(sig) && opt.cert_policy_url )
       {
 	if(sig->version<4)
-	  log_info(_("can't put a policy URL into v3 key (PGP 2.x style) "
-		     "signatures\n"));
+	  log_error(_("can't put a policy URL into v3 key (PGP 2.x style) "
+		      "signatures\n"));
 	else
 	  pu=opt.cert_policy_url;
       }
@@ -163,9 +166,6 @@ mk_notation_and_policy( PKT_signature *sig,
 
 	m_free(s);
       }
-
-    if(opt.show_policy_url)
-      show_policy_url(sig,0);
 }
 
 
@@ -266,6 +266,12 @@ do_sign( PKT_secret_key *sk, PKT_signature *sig,
 	    return G10ERR_TIME_CONFLICT;
     }
 
+    /* For safety, only allow revocation sigs from Elgamal
+       sign+encrypt keys.  Note that this allows for Elgamal
+       designated revocations as well, but that's arguably a good
+       thing. */
+    if(is_ELGAMAL(sk->pubkey_algo) && sig->sig_class!=0x20)
+      return G10ERR_UNU_SECKEY;
 
     print_pubkey_algo_note(sk->pubkey_algo);
 
@@ -460,20 +466,25 @@ write_plaintext_packet (IOBUF out, IOBUF inp, const char *fname, int ptmode)
     }
 
     /* try to calculate the length of the data */
-    if (fname) {
-        if( !(filesize = iobuf_get_filelength(inp)) )
+    if (fname && *fname && !(*fname=='-' && !fname[1])) {
+        off_t tmpsize;
+       
+        if( !(tmpsize = iobuf_get_filelength(inp)) )
             log_info (_("WARNING: `%s' is an empty file\n"), fname);
 
-        /* we can't yet encode the length of very large files,
-         * so we switch to partial length encoding in this case */
-        if (filesize >= IOBUF_FILELENGTH_LIMIT)
-            filesize = 0;
+        /* We can't encode the length of very large files because
+           OpenPGP uses only 32 bit for file sizes.  So if the size of
+           a file is larger than 2^32 minus some bytes for packet
+           headers, we switch to partial length encoding. */
+        if ( tmpsize < (IOBUF_FILELENGTH_LIMIT - 65536) )
+          filesize = tmpsize;
+        else
+          filesize = 0;
 
-        /* because the text_filter modifies the length of the
+        /* Because the text_filter modifies the length of the
          * data, it is not possible to know the used length
          * without a double read of the file - to avoid that
-         * we simple use partial length packets.
-         */
+         * we simple use partial length packets. */
         if ( ptmode == 't' )
             filesize = 0;
     }
@@ -487,7 +498,7 @@ write_plaintext_packet (IOBUF out, IOBUF inp, const char *fname, int ptmode)
         pt->timestamp = make_timestamp ();
         pt->mode = ptmode;
         pt->len = filesize;
-        pt->new_ctb = !pt->len && !opt.rfc1991;
+        pt->new_ctb = !pt->len && !RFC1991;
         pt->buf = inp;
         init_packet(&pkt);
         pkt.pkttype = PKT_PLAINTEXT;
@@ -538,7 +549,7 @@ write_signature_packets (SK_LIST sk_list, IOBUF out, MD_HANDLE hash,
 
 	/* build the signature packet */
 	sig = m_alloc_clear (sizeof *sig);
-	if(opt.force_v3_sigs || opt.rfc1991)
+	if(opt.force_v3_sigs || RFC1991)
 	  sig->version=3;
 	else if(duration || opt.sig_policy_url || opt.sig_notation_data)
 	  sig->version=4;
@@ -637,18 +648,17 @@ sign_file( STRLIST filenames, int detached, STRLIST locusr,
     if( fname && filenames->next && (!detached || encryptflag) )
 	log_bug("multiple files can only be detached signed");
 
-    if(opt.ask_sig_expire && !opt.force_v3_sigs && !opt.batch && !opt.rfc1991)
+    if(opt.ask_sig_expire && !opt.force_v3_sigs && !opt.batch && !RFC1991)
       duration=ask_expire_interval(1);
 
     if( (rc=build_sk_list( locusr, &sk_list, 1, PUBKEY_USAGE_SIG )) )
 	goto leave;
 
-    if(opt.pgp2 && !only_old_style(sk_list))
+    if(PGP2 && !only_old_style(sk_list))
       {
 	log_info(_("you can only detach-sign with PGP 2.x style keys "
 		   "while in --pgp2 mode\n"));
-	log_info(_("this message may not be usable by %s\n"),"PGP 2.x");
-	opt.pgp2=0;
+	compliance_failure();
       }
 
     if(encryptflag && (rc=build_pk_list( remusr, &pk_list, PUBKEY_USAGE_ENC )))
@@ -721,7 +731,8 @@ sign_file( STRLIST filenames, int detached, STRLIST locusr,
 		hashlen=20;
 
 	    if((algo=
-		select_algo_from_prefs(pk_list,PREFTYPE_HASH,-1,&hashlen))>0)
+		select_algo_from_prefs(pk_list,PREFTYPE_HASH,-1,
+				       hashlen?&hashlen:NULL))>0)
 	      recipient_digest_algo=algo;
 	  }
       }
@@ -734,7 +745,7 @@ sign_file( STRLIST filenames, int detached, STRLIST locusr,
     if( !multifile )
 	iobuf_push_filter( inp, md_filter, &mfx );
 
-    if( detached && !encryptflag && !opt.rfc1991 )
+    if( detached && !encryptflag && !RFC1991 )
 	afx.what = 2;
 
     if( opt.armor && !outfile  )
@@ -764,23 +775,20 @@ sign_file( STRLIST filenames, int detached, STRLIST locusr,
 		select_algo_from_prefs(pk_list,PREFTYPE_ZIP,-1,NULL))==-1)
 	      compr_algo=DEFAULT_COMPRESS_ALGO;
 	  }
- 	else if(!opt.expert &&
- 		select_algo_from_prefs(pk_list,PREFTYPE_ZIP,
- 				       compr_algo,NULL)!=compr_algo)
+ 	else if(!opt.expert && pk_list
+ 		&& select_algo_from_prefs(pk_list,PREFTYPE_ZIP,
+					  compr_algo,NULL)!=compr_algo)
  	  log_info(_("forcing compression algorithm %s (%d) "
  		     "violates recipient preferences\n"),
  		   compress_algo_to_string(compr_algo),compr_algo);
 
 	/* algo 0 means no compression */
 	if( compr_algo )
-	  {
-	    zfx.algo = compr_algo;
-	    iobuf_push_filter( out, compress_filter, &zfx );
-	  }
+	  push_compress_filter(out,&zfx,compr_algo);
       }
 
     /* Write the one-pass signature packets if needed */
-    if (!detached && !opt.rfc1991) {
+    if (!detached && !RFC1991) {
         rc = write_onepass_sig_packets (sk_list, out,
                                         opt.textmode && !outfile ? 0x01:0x00);
         if (rc)
@@ -874,14 +882,14 @@ clearsign_file( const char *fname, STRLIST locusr, const char *outfile )
     int rc = 0;
     SK_LIST sk_list = NULL;
     SK_LIST sk_rover = NULL;
-    int old_style = opt.rfc1991;
+    int old_style = RFC1991;
     int only_md5 = 0;
     u32 duration=0;
 
     memset( &afx, 0, sizeof afx);
     init_packet( &pkt );
 
-    if(opt.ask_sig_expire && !opt.force_v3_sigs && !opt.batch && !opt.rfc1991)
+    if(opt.ask_sig_expire && !opt.force_v3_sigs && !opt.batch && !RFC1991)
       duration=ask_expire_interval(1);
 
     if( (rc=build_sk_list( locusr, &sk_list, 1, PUBKEY_USAGE_SIG )) )
@@ -890,12 +898,11 @@ clearsign_file( const char *fname, STRLIST locusr, const char *outfile )
     if( !old_style && !duration )
 	old_style = only_old_style( sk_list );
 
-    if(!old_style && opt.pgp2)
+    if(PGP2 && !only_old_style(sk_list))
       {
 	log_info(_("you can only clearsign with PGP 2.x style keys "
 		   "while in --pgp2 mode\n"));
-	log_info(_("this message may not be usable by %s\n"),"PGP 2.x");
-	opt.pgp2=0;
+	compliance_failure();
       }
 
     /* prepare iobufs */
@@ -1022,7 +1029,7 @@ sign_symencrypt_file (const char *fname, STRLIST locusr)
     memset( &cfx, 0, sizeof cfx);
     init_packet( &pkt );
 
-    if(opt.ask_sig_expire && !opt.force_v3_sigs && !opt.batch && !opt.rfc1991)
+    if(opt.ask_sig_expire && !opt.force_v3_sigs && !opt.batch && !RFC1991)
       duration=ask_expire_interval(1);
 
     rc = build_sk_list (locusr, &sk_list, 1, PUBKEY_USAGE_SIG);
@@ -1041,7 +1048,7 @@ sign_symencrypt_file (const char *fname, STRLIST locusr)
 
     /* prepare key */
     s2k = m_alloc_clear( sizeof *s2k );
-    s2k->mode = opt.rfc1991? 0:opt.s2k_mode;
+    s2k->mode = RFC1991? 0:opt.s2k_mode;
     s2k->hash_algo = opt.def_digest_algo ? opt.def_digest_algo
 	                                 : opt.s2k_digest_algo;
 
@@ -1057,6 +1064,13 @@ sign_symencrypt_file (const char *fname, STRLIST locusr)
         goto leave;
     }
 
+    /* We have no way to tell if the recipient can handle messages
+       with an MDC, so this defaults to no.  Perhaps in a few years,
+       this can be defaulted to yes.  Note that like regular
+       encrypting, --force-mdc overrides --disable-mdc. */
+    if(opt.force_mdc)
+      cfx.dek->use_mdc=1;
+
     /* now create the outfile */
     rc = open_outfile (fname, opt.armor? 1:0, &out);
     if (rc)
@@ -1080,7 +1094,7 @@ sign_symencrypt_file (const char *fname, STRLIST locusr)
 
     /* Write the symmetric key packet */
     /*(current filters: armor)*/
-    if (!opt.rfc1991) {
+    if (!RFC1991) {
 	PKT_symkey_enc *enc = m_alloc_clear( sizeof *enc );
 	enc->version = 4;
 	enc->cipher_algo = cfx.dek->algo;
@@ -1097,23 +1111,11 @@ sign_symencrypt_file (const char *fname, STRLIST locusr)
 
     /* Push the Zip filter */
     if (opt.compress)
-      {
-	int compr_algo=opt.def_compress_algo;
-
-	/* Default */
-        if(compr_algo==-1)
-	  compr_algo=DEFAULT_COMPRESS_ALGO;
-
-	if (compr_algo)
-	  {
-	    zfx.algo = compr_algo;
-	    iobuf_push_filter( out, compress_filter, &zfx );
-	  }
-      }
+      push_compress_filter(out,&zfx,opt.def_compress_algo);
 
     /* Write the one-pass signature packets */
     /*(current filters: zip - encrypt - armor)*/
-    if (!opt.rfc1991) {
+    if (!RFC1991) {
         rc = write_onepass_sig_packets (sk_list, out,
                                         opt.textmode? 0x01:0x00);
         if (rc)
@@ -1174,7 +1176,7 @@ make_keysig_packet( PKT_signature **ret_sig, PKT_public_key *pk,
     MD_HANDLE md;
 
     assert( (sigclass >= 0x10 && sigclass <= 0x13) || sigclass == 0x1F
-	    || sigclass == 0x20 || sigclass == 0x18
+	    || sigclass == 0x20 || sigclass == 0x18 || sigclass == 0x19
 	    || sigclass == 0x30 || sigclass == 0x28 );
 
     if (opt.force_v4_certs)
@@ -1213,14 +1215,19 @@ make_keysig_packet( PKT_signature **ret_sig, PKT_public_key *pk,
 
     md = md_open( digest_algo, 0 );
 
-    /* hash the public key certificate and the user id */
+    /* hash the public key certificate */
     hash_public_key( md, pk );
-    if( sigclass == 0x18 || sigclass == 0x28 ) { /* subkey binding/revocation*/
+
+    if( sigclass == 0x18 || sigclass == 0x19 || sigclass == 0x28 )
+      {
+	/* hash the subkey binding/backsig/revocation */
 	hash_public_key( md, subpk );
-    }
-    else if( sigclass != 0x1F && sigclass != 0x20 ) {
+      }
+    else if( sigclass != 0x1F && sigclass != 0x20 )
+      {
+	/* hash the user id */
         hash_uid (md, sigversion, uid);
-    }
+      }
     /* and make the signature packet */
     sig = m_alloc_clear( sizeof *sig );
     sig->version = sigversion;
@@ -1276,8 +1283,7 @@ update_keysig_packet( PKT_signature **ret_sig,
                       PKT_public_key *subpk,
                       PKT_secret_key *sk,
                       int (*mksubpkt)(PKT_signature *, void *),
-                      void *opaque
-		   )
+                      void *opaque )
 {
     PKT_signature *sig;
     int rc=0;
@@ -1300,7 +1306,7 @@ update_keysig_packet( PKT_signature **ret_sig,
 
     /* create a new signature packet */
     sig = copy_signature (NULL, orig_sig);
-
+ 
     /* We need to create a new timestamp so that new sig expiration
        calculations are done correctly... */
     sig->timestamp=make_timestamp();
