@@ -1,5 +1,5 @@
 /* fileutil.c -  file utilities
- *	Copyright (C) 1998 Free Software Foundation, Inc.
+ *	Copyright (C) 1998, 2003 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -73,10 +73,10 @@ make_dirname(const char *filepath)
     char *p;
 
     if ( !(p=strrchr(filepath, DIRSEP_C)) )
-      #ifdef HAVE_DRIVE_LETTERS
+#ifdef HAVE_DRIVE_LETTERS
 	if ( !(p=strrchr(filepath, '\\')) )
 	    if ( !(p=strrchr(filepath, ':')) )
-      #endif
+#endif
 	      {
 		return m_strdup(EXTSEP_S);
 	      }
@@ -143,11 +143,11 @@ compare_filenames( const char *a, const char *b )
      * resolve symlinks?
      */
 #ifndef __riscos__
-  #ifdef HAVE_DRIVE_LETTERS
+#ifdef HAVE_DRIVE_LETTERS
     return ascii_strcasecmp(a,b);
-  #else
+#else
     return strcmp(a,b);
-  #endif
+#endif
 #else /* __riscos__ */
     int c = 0;
     char *abuf, *bbuf;
@@ -205,7 +205,7 @@ is_file_compressed( const char *s, int *ret_rc )
         { 4, { 0x50, 0x4b, 0x03, 0x04 } }, /* (pk)zip */
     };
     
-    if ( !s || *s == '-' || !ret_rc )
+    if ( !s || (*s == '-' && !s[1]) || !ret_rc )
         return 0; /* We can't check stdin or no file was given */
 
     a = iobuf_open( s );
@@ -236,5 +236,3 @@ leave:
     iobuf_close( a );
     return rc;
 }
-
-
